
/////////////////////////////////////////////////////////////////////
// Mapper 6
void NES_mapper6::Reset()
{
  // set CPU bank pointers
  set_CPU_banks(0,1,14,15);

  // set PPU bank pointers
  if(num_1k_VROM_banks)
  {
    set_PPU_banks(0,1,2,3,4,5,6,7);
  }
  else
  {
    parent_NES->ppu->PPU_VRAM_banks[0] = chr_ram + 0*0x400;
    parent_NES->ppu->PPU_VRAM_banks[1] = chr_ram + 1*0x400;
    parent_NES->ppu->PPU_VRAM_banks[2] = chr_ram + 2*0x400;
    parent_NES->ppu->PPU_VRAM_banks[3] = chr_ram + 3*0x400;
    parent_NES->ppu->PPU_VRAM_banks[4] = chr_ram + 4*0x400;
    parent_NES->ppu->PPU_VRAM_banks[5] = chr_ram + 5*0x400;
    parent_NES->ppu->PPU_VRAM_banks[6] = chr_ram + 6*0x400;
    parent_NES->ppu->PPU_VRAM_banks[7] = chr_ram + 7*0x400;
  }
}

void NES_mapper6::MemoryWriteLow(uint32 addr, uint8 data)
{
  switch(addr)
  {
    case 0x42FE:
      {
        if(data & 0x10)
        {
          set_mirroring(1,1,1,1);
        }
        else
        {
          set_mirroring(0,0,0,0);
        }
      }
      break;

    case 0x42FF:
      {
        if(data & 0x10)
        {
          set_mirroring(NES_PPU::MIRROR_HORIZ);
        }
        else
        {
          set_mirroring(NES_PPU::MIRROR_VERT);
        }
      }
      break;

    case 0x4501:
      {
        irq_enabled = 0;
      }
      break;

    case 0x4502:
      {
        irq_counter = (irq_counter & 0xFF00) | (uint32)data;
      }
      break;

    case 0x4503:
      {
        irq_counter = (irq_counter & 0x00FF) | ((uint32)data << 8);
        irq_enabled = 1;
      }
      break;
  }
}

void NES_mapper6::MemoryWrite(uint32 addr, uint8 data)
{
  uint8 prg_bank = (data & 0x3C) >> 2;
  uint8 chr_bank = data & 0x03;

  set_CPU_bank4(prg_bank*2+0);
  set_CPU_bank5(prg_bank*2+1);

  parent_NES->ppu->PPU_VRAM_banks[0] = chr_ram + chr_bank*0x2000 + 0*0x400;
  parent_NES->ppu->PPU_VRAM_banks[1] = chr_ram + chr_bank*0x2000 + 1*0x400;
  parent_NES->ppu->PPU_VRAM_banks[2] = chr_ram + chr_bank*0x2000 + 2*0x400;
  parent_NES->ppu->PPU_VRAM_banks[3] = chr_ram + chr_bank*0x2000 + 3*0x400;
  parent_NES->ppu->PPU_VRAM_banks[4] = chr_ram + chr_bank*0x2000 + 4*0x400;
  parent_NES->ppu->PPU_VRAM_banks[5] = chr_ram + chr_bank*0x2000 + 5*0x400;
  parent_NES->ppu->PPU_VRAM_banks[6] = chr_ram + chr_bank*0x2000 + 6*0x400;
  parent_NES->ppu->PPU_VRAM_banks[7] = chr_ram + chr_bank*0x2000 + 7*0x400;
}

void NES_mapper6::HSync(uint32 scanline)
{
  if(irq_enabled)
  {
    irq_counter += 133;
    if(irq_counter >= 0xFFFF)
    {
      parent_NES->cpu->DoIRQ();
      irq_counter = 0;
    }
  }
}
/////////////////////////////////////////////////////////////////////

